@@ -200,9 +200,13 @@ class MenuExtension_ownCloud(GObject.GObject, Nemo.MenuProvider):
                 break
         return (topLevelFolder, internalFile)
 
-    def get_file_items(self, window, files):
+    # args in Nemo 4.0: [files: List[Nemo.FileInfo]]
+    # args in Nemo 3.0: [window: Gtk.Widget, files: List[Nemo.FileInfo]]
+    # args[-1] is then compatible with both APIs
+    def get_file_items(self, *args):
         # Show the menu extension to share a file or folder
 
+        files = args[-1]
         # Get usable file paths from the uris
         all_internal_files = True
         for i, file_uri in enumerate(files):
@@ -237,44 +241,65 @@ class MenuExtension_ownCloud(GObject.GObject, Nemo.MenuProvider):
 
         done = False
         start = time.time()
-        timeout = 0.1 # 100ms
+        # timeout is specified in seconds
+        timeout = 0.5
         menu_items = []
         while not done:
             dt = time.time() - start
+
             if dt >= timeout:
                 break
+
             if not socketConnect.read_socket_data_with_timeout(timeout - dt):
                 break
+
             for line in socketConnect.get_available_responses():
-                # Process lines we don't care about
-                if done or not (line.startswith('GET_MENU_ITEMS:') or line.startswith('MENU_ITEM:')):
-                    socketConnect.handle_server_response(line)
-                    continue
-                if line == 'GET_MENU_ITEMS:END':
-                    done = True
-                    # don't break - we'd discard other responses
-                if line.startswith('MENU_ITEM:'):
-                    args = line.split(':')
-                    if len(args) < 4:
+                # if we're done with the menu items, we don't have to try to parse lines any more
+                if not done:
+                    # using a bool to keep track of this eliminates duplicate code when parsing succeeded
+                    line_handled = False
+
+                    if line == 'GET_MENU_ITEMS:END':
+                        done = True
+                        line_handled = True
+
+                    elif line.startswith('MENU_ITEM:'):
+                        args = line.split(':')
+                        if len(args) >= 4:
+                            action = args[1]
+                            label = ':'.join(args[3:])
+                            enabled = 'd' not in args[2]
+
+                            item = Nemo.MenuItem(name=action, label=label, sensitive=enabled)
+                            item.connect("activate", self.context_menu_action, action, filesstring)
+
+                            menu_items.append(item)
+
+                            line_handled = True
+
+                    if line_handled:
+                        # we don't have to have call the handler any more below
                         continue
-                    menu_items.append([args[1], 'd' not in args[2], ':'.join(args[3:])])
+
+                # original comment: Process lines we don't care about
+                socketConnect.handle_server_response(line)
 
         if not done:
             return self.legacy_menu_items(files)
 
-        if len(menu_items) == 0:
+        # if there are no items for the submenu, we don't have to create it at all
+        if not menu_items:
             return []
 
         # Set up the 'ownCloud...' submenu
         item_owncloud = Nemo.MenuItem(
             name='IntegrationMenu', label=self.strings.get('CONTEXT_MENU_TITLE', appname))
-        menu = Nemo.Menu()
-        item_owncloud.set_submenu(menu)
 
-        for action, enabled, label in menu_items:
-            item = Nemo.MenuItem(name=action, label=label, sensitive=enabled)
-            item.connect("activate", self.context_menu_action, action, filesstring)
-            menu.append_item(item)
+        submenu = Nemo.Menu()
+        item_owncloud.set_submenu(submenu)
+
+        for item in menu_items:
+            submenu.append_item(item)
 
         return [item_owncloud]
 
